"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ElectronMessageHub = void 0;
const duplex_message_1 = require("duplex-message");
class ElectronMessageHub extends duplex_message_1.AbstractHub {
    constructor(options) {
        super();
        options = Object.assign({ channelName: 'message-hub' }, options);
        this._channelName = options.channelName;
        this._responseCallbacks = [];
        this._onMessageReceived = this._onMessageReceived.bind(this);
    }
    sendMessage(target, msg) {
        let newMsg = msg;
        if (this._isRequest(msg)) {
            const options = msg.args[0];
            if (options && typeof options.onprogress === 'function') {
                newMsg = Object.assign({}, msg, { progress: true });
                newMsg.args = newMsg.args.slice();
                const copied = Object.assign({}, options);
                delete copied.onprogress;
                newMsg.args[0] = copied;
            }
        }
        target.send(this._channelName, newMsg);
    }
    listenResponse(target, reqMsg, callback) {
        const options = reqMsg.args[0];
        const onprogress = options && typeof options.onprogress === 'function' && options.onprogress;
        let progressName = onprogress ? `${reqMsg.messageID}-progress` : '';
        const eventCallback = (response) => {
            if (!callback(response))
                return false;
            if (progressName)
                this._ipc.removeAllListeners(progressName);
            return true;
        };
        this._responseCallbacks.push(eventCallback);
        if (!progressName)
            return;
        this._ipc.on(progressName, (evt, resp) => {
            onprogress(resp);
        });
    }
    async _onMessageReceived(evt, msg) {
        if (!msg)
            return;
        if (!this._isRequest(msg)) {
            const idx = this._responseCallbacks.findIndex(fn => fn(msg));
            if (idx >= 0)
                this._responseCallbacks.splice(idx, 1);
            return;
        }
        if (msg.progress && msg.args[0]) {
            msg.args[0].onprogress = (data) => {
                evt.sender.send(`${msg.messageID}-progress`, data);
            };
        }
        let response;
        try {
            response = await this.onRequest(evt.sender, msg);
        }
        catch (error) {
            response = error;
        }
        this.sendMessage(evt.sender, response);
    }
}
exports.ElectronMessageHub = ElectronMessageHub;
